$errorA = forceRequiredAddOn("GameMode_Slayer");
$errorB = forceRequiredAddOn("Weapon_Gun");

if($errorA == $Error::AddOn_NotFound)
{
	error("ERROR: Gamemode_Gun_Game - required add-on GameMode_Slayer not found.");

	deleteVariables("$errorA");
	deleteVariables("$errorB");

	return;
}

else if($errorA == $Error::AddOn_Disabled)
{
	error("ERROR: Gamemode_Gun_Game - required add-on GameMode_Slayer has been enabled by force.");
}

if($errorB == $Error::AddOn_Disabled)
{
	error("ERROR: Gamemode_Gun_Game - required add-on Weapon_Gun has been enabled by force.");
}

deleteVariables("$errorA");
deleteVariables("$errorB");

datablock PlayerData(GunGamePlayer : PlayerStandardArmor)
{
	canJet = 0;

	uiName = "Gun Game Player";

	maxTools = 2;
	maxWeapons = 2;
};

$Pref::Server::GunGame::RandomItemBlackList = "fillcanItem invisiCanItem";
//Item datablock names. Space deliminated. Requires minigame reset for changes to take affect. Most non-weapons are automatically blacklisted.

exec("./support.cs");
exec("./slayerRegistry.cs");

function gameConnection::isInGunGameMini(%client)
{
	if(!isObject(%mini = %client.minigame) || !isObject(%mini.gameMode) || %mini.gameMode.class !$= "GunGame")
	{
		return false;
	}

	return %mini;
}

function miniGameSO::getNumGunGameLevels(%mini)
{
	%found = 0;

	for(%i = 0; %i < %mini.gunGameNumLevels; %i++)
	{
		if(%mini.gunGameLevel[%i] $= "")
		{
			break;
		}

		%found++;
	}

	return %found;
}

function gameConnection::updateGunGameEquipment(%client, %silent)
{
	if(!isObject(%player = %client.player) || !%mini = %client.isInGunGameMini())
	{
		return;
	}

	if(%player.tool0 != %mini.gunGameLevel[%client.gunGameLevel])
	{
		if(!isObject(%player.tool0) && %mini.gunGameLevel[%client.gunGameLevel].className $= "Weapon")
		{
			%player.weaponCount++;
		}

		%player.tool0 = %mini.gunGameLevel[%client.gunGameLevel];
		messageClient(%client, 'MsgItemPickup', "", 0, %player.tool0, %silent);
	}

	if(%player.tool1 != %mini.gunGameDemoteWep)
	{
		if(!isObject(%player.tool1) && %mini.gunGameDemoteWep.className $= "Weapon")
		{
			%player.weaponCount++;
		}

		%player.tool1 = %mini.gunGameDemoteWep;
		messageClient(%client, 'MsgItemPickup', "", 1, %player.tool1, %silent);
	}

	if(%player.currTool $= "0")
	{
		%player.mountImage(%player.tool0.image, %player.tool0.image.mountPoint);
	}

	else if(%player.currTool == 1)
	{
		%player.mountImage(%player.tool1.image, %player.tool1.image.mountPoint);
	}
}

function serverCmdGunList(%client)
{
	serverCmdWeaponList(%client);
}

function serverCmdWepList(%client)
{
	serverCmdWeaponList(%client);
}

function serverCmdWeaponList(%client)
{
	if(!%mini = %client.isInGunGameMini())
	{
		return;
	}

	for(%i = 0; %i <= %mini.getNumGunGameLevels(); %i++)
	{
		%primeColor = %i == %client.gunGameLevel ? "\c2" : "\c3";

		messageClient(%client, '', "\c6[" @ %primeColor @ %i @ "\c6]" @ %primeColor SPC %mini.gunGameLevel[%i].uiName);
	}
}

function GunGame::promoteClient(%mode, %client, %silent)
{
	if(!isObject(%client) || !%client.isInGunGameMini())
	{
		return;
	}

	%client.gunGameLevel++;

	if(%client.gunGameLevel > %client.minigame.getNumGunGameLevels())
	{
		%client.gunGameLevel--;
		%client.minigame.endRound(%client);

		return;
	}

	commandToClient(%client, 'centerPrint', "<just:right>\c6Level:\c2" SPC %client.gunGameLevel @ "\c6!" NL "\c3" @ %client.minigame.gunGameLevel[%client.gunGameLevel].uiName, 3);

	%client.updateGunGameEquipment(%silent);
}

function GunGame::demoteClient(%mode, %client, %silent)
{
	if(!isObject(%client) || !%client.isInGunGameMini())
	{
		return;
	}

	if(!%client.gunGameLevel)
	{
		return;
	}

	%client.gunGameLevel--;

	commandToClient(%client, 'centerPrint', "<just:right>\c6Level:\c0" SPC %client.gunGameLevel @ "\c6." NL "\c3" @ %client.minigame.gunGameLevel[%client.gunGameLevel].uiName, 3);

	%client.updateGunGameEquipment(%silent);
}

function GunGame::onGameModeStart(%mode)
{
	for(%i = 0; %i < %mode.minigame.numMembers; %i++)
	{
		%mode.minigame.member[%i].gunGameLevel = 0;
	}
}

function GunGame::onGameModeEnd(%mode)
{
	for(%i = 0; %i < %mode.minigame.numMembers; %i++)
	{
		%mode.minigame.member[%i].gunGameLevel = "";
	}
}

function GunGame::onClientJoinGame(%mode, %client)
{
	%client.gunGameLevel = 0;
}

function GunGame::onClientLeaveGame(%mode, %client)
{
	%client.gunGameLevel = "";
}

function GunGame::onMinigameReset(%mode, %client)
{
	%mini = %mode.minigame;

	for(%i = 0; %i < %mini.numMembers; %i++)
	{
		%mini.member[%i].gunGameLevel = 0;
	}

	if(isObject(GunGame_MeleeItemCache) && !(GunGame_MeleeItemCache.itemCount || $Pref::Server::GunGame::RandomItemBlackList !$= $GunGame::LastBlackList))
	{
		GunGame_MeleeItemCache.delete();
	}

	if(isObject(GunGame_ItemCache) && (!GunGame_ItemCache.itemCount || $Pref::Server::GunGame::RandomItemBlackList !$= $GunGame::LastBlackList))
	{
		GunGame_ItemCache.delete();
	}

	if($Pref::Server::GunGame::RandomItemBlackList !$= $GunGame::LastBlackList)
	{
		$GunGame::LastBlackList = $Pref::Server::GunGame::RandomItemBlackList;
	}

	if(!%mini.gunGameMixDemote && !%mini.gunGameMixLevels)
	{
		return;
	}

	if(!isObject(GunGame_MeleeItemCache))
	{
		%fillMeleeItemCache = true;

		new scriptObject(GunGame_MeleeItemCache)
		{
			itemCount = 0;
		};
	}

	if(!isObject(GunGame_ItemCache))
	{
		%fillItemCache = true;

		new scriptObject(GunGame_ItemCache)
		{
			itemCount = 0;
		};
	}

	if(%fillMeleeItemCache || %fillItemCache)
	{
		%xCount = getDataBlockGroupSize();

		for(%x = 0; %x < %xCount; %x++)
		{
			%obj = getDataBlock(%x);

			if(!isObject(%obj) || %obj.getClassName() !$= "ItemData" || %obj.uiName $= "" || %obj.category !$= "Weapon" || %obj.className !$= "Weapon" || !isObject(%obj.image) || hasItemOnList($GunGame::RandomItemBlackList, %obj.getName()))
			{
				continue;
			}

			if(%fillMeleeItemCache && %obj.isMelee())
			{
				GunGame_MeleeItemCache.item[GunGame_MeleeItemCache.itemCount] = %obj.getID();
				GunGame_MeleeItemCache.itemCount++;
			}

			if(%fillItemCache)
			{
				GunGame_ItemCache.item[GunGame_ItemCache.itemCount] = %obj.getID();
				GunGame_ItemCache.itemCount++;
			}
		}
	}

	if(%mini.gunGameMixDemote)
	{
		if(%mini.gunGameMixDemote == 1 && GunGame_MeleeItemCache.itemCount)	//Pick a melee weapon for the random
		{			//secondary weapon if possible.
			%demoteItem = GunGame_MeleeItemCache.item[getRandom(0, GunGame_MeleeItemCache.itemCount - 1)];
		}

		else //Pick a random item for the secondary weapon.
		{
			%demoteItem = GunGame_ItemCache.item[getRandom(0, GunGame_ItemCache.itemCount - 1)];
		}

		Slayer.Prefs.getPrefSO("Game Mode", "Demote Weapon").setValue(%demoteItem, %mini);
	}

	if(%mini.gunGameMixLevels == 1) //Pick any random items. Prefer unique.
	{
		%count = %mini.getNumGunGameLevels();

		for(%i = 0; %i < %count; %i++)
		{
			if(!isObject(%pref = Slayer.Prefs.getPrefSO("Game Mode", "Level" SPC %i)))
			{
				break;
			}

			%list = addItemToList(%list, GunGame_ItemCache.item[%i]);
		}

		for(%i = 0; %i < %count; %i++)
		{
			if(!isObject(%pref = Slayer.Prefs.getPrefSO("Game Mode", "Level" SPC %i)))
			{
				break;
			}

			if(%list !$= "")
			{
				%item = getWord(%list, getRandom(0, getWordCount(%list) - 1));
				%list = removeItemFromList(%list, %item);
				//Pick a unique random item from the list.
			}

			else
			{
				%item = GunGame_ItemCache.item[getRandom(0, GunGame_ItemCache.itemCount - 1)];
				//Pick a random item if there's no unique ones left.
			}

			%pref.setValue(%item, %mini);
		}

		%list = "";
	}

	else if(%mini.gunGameMixLevels == 2) //Mix the current list of levels.
	{
		%count = %mini.getNumGunGameLevels();

		for(%i = 0; %i < %count; %i++)
		{
			if(!isObject(%pref = Slayer.Prefs.getPrefSO("Game Mode", "Level" SPC %i)))
			{
				break;
			}

			%list = addItemToList(%list, %mini.gunGameLevel[%i]);
		}

		%count = getWordCount(%list);

		for(%i = 0; %i < %count; %i++)
		{
			if(!isObject(%pref = Slayer.Prefs.getPrefSO("Game Mode", "Level" SPC %i)))
			{
				break;
			}

			%item = getWord(%list, getRandom(0, getWordCount(%list) - 1));
			%list = removeItemFromList(%list, %item);

			%pref.setValue(%item, %mini);
		}
	}
}

function GunGame::onPlayerSpawn(%mode, %client)
{
	%client.schedule(0, updateGunGameEquipment, 1);
}

function GunGame::minigameCanUse(%mode, %objA, %objB)
{
	if(%objA.getClassName() $= "Player" && %objB.getClassName() $= "Item")
	{
		return false;
	}

	if(%objB.getClassName() $= "Player" && %objA.getClassName() $= "Item")
	{
		return false;
	}

	return -1; //Let Slayer handle it.
}

function GunGame::onPlayerDeath(%mode, %client, %obj, %killer, %type, %area)
{
	if(%client == %killer || %type == $DamageType::Fall || %type == $DamageType::Impact || %type == $DamageType::Lava || %type == $DamageType::Suicide)
	{
		%mode.demoteClient(%killer, 0);

		return;
	}

	if(%type == %mode.minigame.gunGameLevel[%killer.gunGameLevel].image.projectile.directDamageType || %type == %mode.minigame.gunGameLevel[%killer.gunGameLevel].image.projectile.radiusDamageType)
	{
		%mode.promoteClient(%killer, 0);

		return;
	}

	if(isObject(%mode.minigame.gunGameDemoteWep) && isObject(%mode.minigame.gunGameDemoteWep.image) && isObject(%mode.minigame.gunGameDemoteWep.image.projectile) && (%type == %mode.minigame.gunGameDemoteWep.image.projectile.directDamageType || %type == %mode.minigame.gunGameDemoteWep.image.projectile.radiusDamageType))
	{
		%mode.demoteClient(%client, 0);
	}
}

function GunGame::victoryCheck_time(%mode)
{
	%mini = %mode.minigame;

	if(%mini.timeElapsed < %mini.time * 60000)
	{
		return 0;
	}

	for(%i = 0; %i < %mini.numMembers; %i++)
	{
		%client = %mini.member[%i];

		if(%client.gunGameLevel > %least)
		{
			%winner = %client;
			%least = %client.gunGameLevel;
			%count = 1;
		}

		else if(%client.gunGameLevel == %least && %least > 0)
		{
			%winner = setField(%winner, %count, %client);
			%count++;
		}
	}

	return %winner;
}

function GunGame::scoreListInit(%mode, %header, %var1, %var2, %var3, %var4, %var5, %var6, %var7, %var8, %var9)
{
	return true TAB '<color:ffffff><tab:142, 217, 349, 394, 469><font:arial:16>%1<h2>%2\t%3\t%4\t%5\t%6\t%7</h2><br>' TAB %var1 TAB %var2 TAB %var3 TAB "Weapon" TAB %var4 TAB %var5 TAB %var6;
}

function GunGame::scoreListAdd(%mode, %obj, %line, %var1, %var2, %var3, %var4, %var5, %var6, %var7, %var8, %var9)
{
	return true TAB '<color:%1><b>%2</b></color>\t%3\t%4\t%5\t%6\t%7\t%8<br>' TAB %var1 TAB %var2 TAB %var3 TAB %mode.minigame.gunGameLevel[%obj.gunGameLevel].uiName TAB %var4 TAB %var5 TAB %var6;
}

package GunGame
{
	function serverCmdDropTool(%client, %slot)
	{
		if(%client.isInGunGameMini())
		{
			messageClient(%client, '', "\c5Dropping items is disabled in this minigame.");

			return;
		}

		parent::serverCmdDropTool(%client, %slot);
	}
};
activatePackage(GunGame);